@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2018-2022 Mike Fährmann
+# Copyright 2018-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -10,6 +10,7 @@
 
 from .common import DownloaderBase
 from .. import ytdl, text
+from xml.etree import ElementTree
 import os
 
 
@@ -29,6 +30,7 @@ class YoutubeDLDownloader(DownloaderBase):
         }
 
         self.ytdl_instance = None
+        self.rate_dyn = None
         self.forward_cookies = self.config("forward-cookies", True)
         self.progress = self.config("progress", 3.0)
         self.outtmpl = self.config("outtmpl")
@@ -48,6 +50,13 @@ class YoutubeDLDownloader(DownloaderBase):
                     self.log.debug("", exc_info=exc)
                     self.download = lambda u, p: False
                     return False
+
+                try:
+                    ytdl_version = module.version.__version__
+                except Exception:
+                    ytdl_version = ""
+                self.log.debug("Using %s version %s", module, ytdl_version)
+
                 self.ytdl_instance = ytdl_instance = ytdl.construct_YoutubeDL(
                     module, self, self.ytdl_opts)
                 if self.outtmpl == "default":
@@ -59,8 +68,13 @@ class YoutubeDLDownloader(DownloaderBase):
                 for cookie in self.session.cookies:
                     set_cookie(cookie)
 
-        if self.progress is not None and not ytdl_instance._progress_hooks:
-            ytdl_instance.add_progress_hook(self._progress_hook)
+        if "__gdl_initialize" in ytdl_instance.params:
+            del ytdl_instance.params["__gdl_initialize"]
+
+            if self.progress is not None:
+                ytdl_instance.add_progress_hook(self._progress_hook)
+            if rlf := ytdl_instance.params.pop("__gdl_ratelimit_func", False):
+                self.rate_dyn = rlf
 
         info_dict = kwdict.pop("_ytdl_info_dict", None)
         if not info_dict:
@@ -69,7 +83,9 @@ class YoutubeDLDownloader(DownloaderBase):
                 manifest = kwdict.pop("_ytdl_manifest", None)
                 if manifest:
                     info_dict = self._extract_manifest(
-                        ytdl_instance, url, manifest)
+                        ytdl_instance, url, manifest,
+                        kwdict.pop("_ytdl_manifest_data", None),
+                        kwdict.pop("_ytdl_manifest_headers", None))
                 else:
                     info_dict = self._extract_info(ytdl_instance, url)
             except Exception as exc:
@@ -121,18 +137,32 @@ class YoutubeDLDownloader(DownloaderBase):
         if pathfmt.exists():
             pathfmt.temppath = ""
             return True
-        if self.part and self.partdir:
-            pathfmt.temppath = os.path.join(
-                self.partdir, pathfmt.filename)
 
-        self._set_outtmpl(ytdl_instance, pathfmt.temppath.replace("%", "%%"))
+        if self.rate_dyn is not None:
+            # static ratelimits are set in ytdl.construct_YoutubeDL
+            ytdl_instance.params["ratelimit"] = self.rate_dyn()
 
         self.out.start(pathfmt.path)
+        if self.part:
+            pathfmt.kwdict["extension"] = pathfmt.prefix
+            filename = pathfmt.build_filename(pathfmt.kwdict)
+            pathfmt.kwdict["extension"] = info_dict["ext"]
+            if self.partdir:
+                path = os.path.join(self.partdir, filename)
+            else:
+                path = pathfmt.realdirectory + filename
+            path = path.replace("%", "%%") + "%(ext)s"
+        else:
+            path = pathfmt.realpath.replace("%", "%%")
+
+        self._set_outtmpl(ytdl_instance, path)
         try:
             ytdl_instance.process_info(info_dict)
         except Exception as exc:
             self.log.debug("", exc_info=exc)
             return False
+
+        pathfmt.temppath = info_dict.get("filepath") or info_dict["_filename"]
         return True
 
     def _download_playlist(self, ytdl_instance, pathfmt, info_dict):
@@ -141,43 +171,67 @@ class YoutubeDLDownloader(DownloaderBase):
         self._set_outtmpl(ytdl_instance, pathfmt.realpath)
 
         for entry in info_dict["entries"]:
+            if self.rate_dyn is not None:
+                ytdl_instance.params["ratelimit"] = self.rate_dyn()
             ytdl_instance.process_info(entry)
         return True
 
     def _extract_info(self, ytdl, url):
         return ytdl.extract_info(url, download=False)
 
-    def _extract_manifest(self, ytdl, url, manifest):
+    def _extract_manifest(self, ytdl, url, manifest_type, manifest_data=None,
+                          headers=None):
         extr = ytdl.get_info_extractor("Generic")
         video_id = extr._generic_id(url)
 
-        if manifest == "hls":
-            try:
-                formats, subtitles = extr._extract_m3u8_formats_and_subtitles(
-                    url, video_id, "mp4")
-            except AttributeError:
-                formats = extr._extract_m3u8_formats(url, video_id, "mp4")
-                subtitles = None
+        if manifest_type == "hls":
+            if manifest_data is None:
+                try:
+                    fmts, subs = extr._extract_m3u8_formats_and_subtitles(
+                        url, video_id, "mp4", headers=headers)
+                except AttributeError:
+                    fmts = extr._extract_m3u8_formats(
+                        url, video_id, "mp4", headers=headers)
+                    subs = None
+            else:
+                try:
+                    fmts, subs = extr._parse_m3u8_formats_and_subtitles(
+                        url, video_id, "mp4")
+                except AttributeError:
+                    fmts = extr._parse_m3u8_formats(url, video_id, "mp4")
+                    subs = None
 
-        elif manifest == "dash":
-            try:
-                formats, subtitles = extr._extract_mpd_formats_and_subtitles(
-                    url, video_id)
-            except AttributeError:
-                formats = extr._extract_mpd_formats(url, video_id)
-                subtitles = None
+        elif manifest_type == "dash":
+            if manifest_data is None:
+                try:
+                    fmts, subs = extr._extract_mpd_formats_and_subtitles(
+                        url, video_id, headers=headers)
+                except AttributeError:
+                    fmts = extr._extract_mpd_formats(
+                        url, video_id, headers=headers)
+                    subs = None
+            else:
+                if isinstance(manifest_data, str):
+                    manifest_data = ElementTree.fromstring(manifest_data)
+                try:
+                    fmts, subs = extr._parse_mpd_formats_and_subtitles(
+                        manifest_data, mpd_id="dash")
+                except AttributeError:
+                    fmts = extr._parse_mpd_formats(
+                        manifest_data, mpd_id="dash")
+                    subs = None
 
         else:
-            self.log.error("Unsupported manifest type '%s'", manifest)
+            self.log.error("Unsupported manifest type '%s'", manifest_type)
             return None
 
         info_dict = {
+            "extractor": "",
             "id"       : video_id,
             "title"    : video_id,
-            "formats"  : formats,
-            "subtitles": subtitles,
+            "formats"  : fmts,
+            "subtitles": subs,
         }
-        #  extr._extra_manifest_info(info_dict, url)
         return ytdl.process_ie_result(info_dict, download=False)
 
     def _progress_hook(self, info):
@@ -191,8 +245,7 @@ class YoutubeDLDownloader(DownloaderBase):
                 int(speed) if speed else 0,
             )
 
-    @staticmethod
-    def _set_outtmpl(ytdl_instance, outtmpl):
+    def _set_outtmpl(self, ytdl_instance, outtmpl):
         try:
             ytdl_instance._parse_outtmpl
         except AttributeError:
