@@ -1,13 +1,13 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2023 Mike Fährmann
+# Copyright 2015-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
 import sys
-import re
+from ..text import re_compile
 
 modules = [
     "2ch",
@@ -24,6 +24,7 @@ modules = [
     "adultempire",
     "agnph",
     "ao3",
+    "arcalive",
     "architizer",
     "artstation",
     "aryion",
@@ -39,12 +40,14 @@ modules = [
     "chevereto",
     "cien",
     "civitai",
-    "cohost",
+    "comick",
     "comicvine",
     "cyberdrop",
     "danbooru",
+    "dankefuerslesen",
     "desktopography",
     "deviantart",
+    "discord",
     "dynastyscans",
     "e621",
     "erome",
@@ -52,22 +55,23 @@ modules = [
     "exhentai",
     "facebook",
     "fanbox",
-    "fanleaks",
     "fantia",
     "fapello",
     "fapachi",
     "flickr",
     "furaffinity",
+    "furry34",
     "fuskator",
     "gelbooru",
     "gelbooru_v01",
     "gelbooru_v02",
+    "girlsreleased",
+    "girlswithmuscle",
     "gofile",
     "hatenablog",
     "hentai2read",
     "hentaicosplays",
     "hentaifoundry",
-    "hentaifox",
     "hentaihand",
     "hentaihere",
     "hentainexus",
@@ -82,50 +86,55 @@ modules = [
     "imgbox",
     "imgth",
     "imgur",
+    "imhentai",
     "inkbunny",
     "instagram",
     "issuu",
     "itaku",
     "itchio",
+    "iwara",
     "jschan",
     "kabeuchi",
     "keenspot",
-    "kemonoparty",
+    "kemono",
     "khinsider",
-    "koharu",
     "komikcast",
     "lensdump",
     "lexica",
     "lightroom",
     "livedoor",
+    "lofter",
     "luscious",
     "lynxchan",
     "mangadex",
     "mangafox",
     "mangahere",
-    "mangakakalot",
     "manganelo",
     "mangapark",
     "mangaread",
-    "mangasee",
     "mangoxo",
     "misskey",
     "motherless",
     "myhentaigallery",
     "myportfolio",
-    "naver",
+    "naverblog",
+    "naverchzzk",
     "naverwebtoon",
+    "nekohouse",
     "newgrounds",
     "nhentai",
     "nijie",
     "nitter",
     "nozomi",
     "nsfwalbum",
+    "nudostar",
     "paheal",
     "patreon",
+    "pexels",
     "philomena",
     "photovogue",
     "picarto",
+    "pictoa",
     "piczel",
     "pillowfort",
     "pinterest",
@@ -138,8 +147,11 @@ modules = [
     "pornhub",
     "pornpics",
     "postmill",
+    "rawkuma",
     "reactor",
     "readcomiconline",
+    "realbooru",
+    "redbust",
     "reddit",
     "redgifs",
     "rule34us",
@@ -148,6 +160,7 @@ modules = [
     "saint",
     "sankaku",
     "sankakucomplex",
+    "schalenetwork",
     "scrolller",
     "seiga",
     "senmanga",
@@ -166,6 +179,8 @@ modules = [
     "tapas",
     "tcbscans",
     "telegraph",
+    "tenor",
+    "tiktok",
     "tmohentai",
     "toyhouse",
     "tsumino",
@@ -188,12 +203,15 @@ modules = [
     "weasyl",
     "webmshare",
     "webtoons",
+    "weebcentral",
     "weibo",
     "wikiart",
     "wikifeet",
     "wikimedia",
+    "xfolio",
     "xhamster",
     "xvideos",
+    "yiffverse",
     "zerochan",
     "zzup",
     "booru",
@@ -224,7 +242,8 @@ def find(url):
 
 def add(cls):
     """Add 'cls' to the list of available extractors"""
-    cls.pattern = re.compile(cls.pattern)
+    if isinstance(cls.pattern, str):
+        cls.pattern = re_compile(cls.pattern)
     _cache.append(cls)
     return cls
 
@@ -232,9 +251,11 @@ def add(cls):
 def add_module(module):
     """Add all extractors in 'module' to the list of available extractors"""
     classes = _get_classes(module)
-    for cls in classes:
-        cls.pattern = re.compile(cls.pattern)
-    _cache.extend(classes)
+    if classes:
+        if isinstance(classes[0].pattern, str):
+            for cls in classes:
+                cls.pattern = re_compile(cls.pattern)
+        _cache.extend(classes)
     return classes
 
 
