@@ -30,7 +30,7 @@ class BilibiliUserArticlesExtractor(BilibiliExtractor):
     def items(self):
         for article in self.api.user_articles(self.groups[0]):
             article["_extractor"] = BilibiliArticleExtractor
-            url = "{}/opus/{}".format(self.root, article["opus_id"])
+            url = f"{self.root}/opus/{article['opus_id']}"
             yield Message.Queue, url, article
 
 
@@ -81,17 +81,38 @@ class BilibiliArticleExtractor(BilibiliExtractor):
             yield Message.Url, url, text.nameext_from_url(url, article)
 
 
+class BilibiliUserArticlesFavoriteExtractor(BilibiliExtractor):
+    subcategory = "user-articles-favorite"
+    pattern = (r"(?:https?://)?space\.bilibili\.com"
+               r"/(\d+)/favlist\?fid=opus")
+    example = "https://space.bilibili.com/12345/favlist?fid=opus"
+    _warning = True
+
+    def _init(self):
+        BilibiliExtractor._init(self)
+        if self._warning:
+            if not self.cookies_check(("SESSDATA",)):
+                self.log.error("'SESSDATA' cookie required")
+            BilibiliUserArticlesFavoriteExtractor._warning = False
+
+    def items(self):
+        for article in self.api.user_favlist():
+            article["_extractor"] = BilibiliArticleExtractor
+            url = f"{self.root}/opus/{article['opus_id']}"
+            yield Message.Queue, url, article
+
+
 class BilibiliAPI():
     def __init__(self, extractor):
         self.extractor = extractor
 
     def _call(self, endpoint, params):
         url = "https://api.bilibili.com/x/polymer/web-dynamic/v1" + endpoint
-        data = self.extractor.request(url, params=params).json()
+        data = self.extractor.request_json(url, params=params)
 
         if data["code"] != 0:
             self.extractor.log.debug("Server response: %s", data)
-            raise exception.StopExtraction("API request failed")
+            raise exception.AbortExtraction("API request failed")
 
         return data
 
@@ -119,6 +140,32 @@ class BilibiliAPI():
                     page, "window.__INITIAL_STATE__=", "};") + "}")
             except Exception:
                 if "window._riskdata_" not in page:
-                    raise exception.StopExtraction(
-                        "%s: Unable to extract INITIAL_STATE data", article_id)
+                    raise exception.AbortExtraction(
+                        f"{article_id}: Unable to extract INITIAL_STATE data")
             self.extractor.wait(seconds=300)
+
+    def user_favlist(self):
+        endpoint = "/opus/feed/fav"
+        params = {"page": 1, "page_size": 20}
+
+        while True:
+            data = self._call(endpoint, params)["data"]
+
+            yield from data["items"]
+
+            if not data.get("has_more"):
+                break
+            params["page"] += 1
+
+    def login_user_id(self):
+        url = "https://api.bilibili.com/x/space/v2/myinfo"
+        data = self.extractor.request_json(url)
+
+        if data["code"] != 0:
+            self.extractor.log.debug("Server response: %s", data)
+            raise exception.AbortExtraction(
+                "API request failed. Are you logges in?")
+        try:
+            return data["data"]["profile"]["mid"]
+        except Exception:
+            raise exception.AbortExtraction("API request failed")
