@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2022-2023 Mike Fährmann
+# Copyright 2022-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -10,7 +10,7 @@
 
 from .common import Extractor
 from .lolisafe import LolisafeAlbumExtractor
-from .. import text, config, exception
+from .. import text, util, config, exception
 import random
 
 if config.get(("extractor", "bunkr"), "tlds"):
@@ -60,6 +60,9 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
     """Extractor for bunkr.si albums"""
     category = "bunkr"
     root = "https://bunkr.si"
+    root_dl = "https://get.bunkrr.su"
+    root_api = "https://apidl.bunkr.ru"
+    archive_fmt = "{album_id}_{id|id_url}"
     pattern = BASE_PATTERN + r"/a/([^/?#]+)"
     example = "https://bunkr.si/a/ID"
 
@@ -69,6 +72,22 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
         if domain not in LEGACY_DOMAINS:
             self.root = "https://" + domain
 
+    def _init(self):
+        LolisafeAlbumExtractor._init(self)
+
+        endpoint = self.config("endpoint")
+        if not endpoint:
+            endpoint = self.root_api + "/api/_001_v2"
+        elif endpoint[0] == "/":
+            endpoint = self.root_api + endpoint
+
+        self.endpoint = endpoint
+        self.offset = 0
+
+    def skip(self, num):
+        self.offset = num
+        return num
+
     def request(self, url, **kwargs):
         kwargs["encoding"] = "utf-8"
         kwargs["allow_redirects"] = False
@@ -105,7 +124,7 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
                     pass
                 else:
                     if not DOMAINS:
-                        raise exception.StopExtraction(
+                        raise exception.AbortExtraction(
                             "All Bunkr domains require solving a CF challenge")
 
             # select alternative domain
@@ -132,6 +151,9 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
         }
 
     def _extract_files(self, items):
+        if self.offset:
+            items = util.advance(items, self.offset)
+
         for item in items:
             try:
                 url = text.unescape(text.extr(item, ' href="', '"'))
@@ -147,30 +169,36 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
                     info[-1], "%H:%M:%S %d/%m/%Y")
 
                 yield file
-            except exception.StopExtraction:
+            except exception.ControlException:
                 raise
             except Exception as exc:
                 self.log.error("%s: %s", exc.__class__.__name__, exc)
                 self.log.debug("", exc_info=exc)
 
     def _extract_file(self, webpage_url):
-        response = self.request(webpage_url)
-        page = response.text
-        file_url = (text.extr(page, '<source src="', '"') or
-                    text.extr(page, '<img src="', '"'))
-        file_name = (text.extr(page, 'property="og:title" content="', '"') or
-                     text.extr(page, "<title>", " | Bunkr<"))
+        page = self.request(webpage_url).text
+        data_id = text.extr(page, 'data-file-id="', '"')
+        referer = self.root_dl + "/file/" + data_id
 
-        if not file_url:
-            webpage_url = text.unescape(text.rextract(
-                page, ' href="', '"', page.rindex("Download"))[0])
-            response = self.request(webpage_url)
-            file_url = text.rextract(response.text, ' href="', '"')[0]
+        headers = {"Referer": referer, "Origin": self.root_dl}
+        data = self.request_json(self.endpoint, method="POST", headers=headers,
+                                 json={"id": data_id})
+
+        if data.get("encrypted"):
+            key = f"SECRET_KEY_{data['timestamp'] // 3600}"
+            file_url = util.decrypt_xor(data["url"], key.encode())
+        else:
+            file_url = data["url"]
+
+        file_name = text.extr(page, "<h1", "<").rpartition(">")[2]
+        fallback = text.extr(page, 'property="og:url" content="', '"')
 
         return {
-            "file"          : text.unescape(file_url),
+            "file"          : file_url,
             "name"          : text.unescape(file_name),
-            "_http_headers" : {"Referer": response.url},
+            "id_url"        : data_id,
+            "_fallback"     : (fallback,) if fallback else (),
+            "_http_headers" : {"Referer": referer},
             "_http_validate": self._validate,
         }
 
