@@ -1,31 +1,30 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2020-2023 Mike Fährmann
+# Copyright 2020-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
-"""Extractors for https://hiperdex.top/"""
+"""Extractors for https://hiperdex.com/"""
 
 from .common import ChapterExtractor, MangaExtractor
-from .. import text
+from .. import text, util
 from ..cache import memcache
-import re
 
 BASE_PATTERN = (r"((?:https?://)?(?:www\.)?"
-                r"(?:1st)?hiperdex\d?\.(?:com|net|info|top))")
+                r"(?:1st)?hiper(?:dex|toon)\d?\.(?:com|net|info|top))")
 
 
 class HiperdexBase():
     """Base class for hiperdex extractors"""
     category = "hiperdex"
-    root = "https://hiperdex.top"
+    root = "https://hiperdex.com"
 
     @memcache(keyarg=1)
     def manga_data(self, manga, page=None):
         if not page:
-            url = "{}/manga/{}/".format(self.root, manga)
+            url = f"{self.root}/manga/{manga}/"
             page = self.request(url).text
         extr = text.extract_from(page)
 
@@ -49,7 +48,7 @@ class HiperdexBase():
             "status" : extr(
                 'class="summary-content">', '<').strip(),
             "description": text.remove_html(text.unescape(extr(
-                'class="description-summary">', '</div>'))),
+                '<div class="description-summary">', "</div>"))),
             "language": "English",
             "lang"    : "en",
         }
@@ -69,7 +68,7 @@ class HiperdexBase():
 class HiperdexChapterExtractor(HiperdexBase, ChapterExtractor):
     """Extractor for hiperdex manga chapters"""
     pattern = BASE_PATTERN + r"(/mangas?/([^/?#]+)/([^/?#]+))"
-    example = "https://hiperdex.top/manga/MANGA/CHAPTER/"
+    example = "https://hiperdex.com/manga/MANGA/CHAPTER/"
 
     def __init__(self, match):
         root, path, self.manga, self.chapter = match.groups()
@@ -80,10 +79,10 @@ class HiperdexChapterExtractor(HiperdexBase, ChapterExtractor):
         return self.chapter_data(self.chapter)
 
     def images(self, page):
+        pattern = util.re(r'id="image-\d+"\s+(?:data-)?src="([^"]+)')
         return [
             (url.strip(), None)
-            for url in re.findall(
-                r'id="image-\d+"\s+(?:data-)?src="([^"]+)', page)
+            for url in pattern.findall(page)
         ]
 
 
@@ -91,7 +90,7 @@ class HiperdexMangaExtractor(HiperdexBase, MangaExtractor):
     """Extractor for hiperdex manga"""
     chapterclass = HiperdexChapterExtractor
     pattern = BASE_PATTERN + r"(/mangas?/([^/?#]+))/?$"
-    example = "https://hiperdex.top/manga/MANGA/"
+    example = "https://hiperdex.com/manga/MANGA/"
 
     def __init__(self, match):
         root, path, self.manga = match.groups()
@@ -100,14 +99,14 @@ class HiperdexMangaExtractor(HiperdexBase, MangaExtractor):
 
     def chapters(self, page):
         data = self.manga_data(self.manga, page)
-        self.manga_url = url = data["url"]
+        self.page_url = url = data["url"]
 
-        url = self.manga_url + "ajax/chapters/"
+        url = self.page_url + "ajax/chapters/"
         headers = {
             "Accept": "*/*",
             "X-Requested-With": "XMLHttpRequest",
             "Origin": self.root,
-            "Referer": "https://" + text.quote(self.manga_url[8:]),
+            "Referer": "https://" + text.quote(self.page_url[8:]),
         }
         html = self.request(url, method="POST", headers=headers).text
 
@@ -127,11 +126,11 @@ class HiperdexArtistExtractor(HiperdexBase, MangaExtractor):
     chapterclass = HiperdexMangaExtractor
     reverse = False
     pattern = BASE_PATTERN + r"(/manga-a(?:rtist|uthor)/(?:[^/?#]+))"
-    example = "https://hiperdex.top/manga-artist/NAME/"
+    example = "https://hiperdex.com/manga-artist/NAME/"
 
     def __init__(self, match):
-        self.root = text.ensure_http_scheme(match.group(1))
-        MangaExtractor.__init__(self, match, self.root + match.group(2) + "/")
+        self.root = text.ensure_http_scheme(match[1])
+        MangaExtractor.__init__(self, match, self.root + match[2] + "/")
 
     def chapters(self, page):
         results = []
