@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2014-2023 Mike Fährmann
+# Copyright 2014-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,14 +8,14 @@
 
 """Extractors for https://www.pixiv.net/"""
 
-from .common import Extractor, Message
+from .common import Extractor, Message, Dispatch
 from .. import text, util, exception
 from ..cache import cache, memcache
 from datetime import datetime, timedelta
 import itertools
 import hashlib
 
-BASE_PATTERN = r"(?:https?://)?(?:www\.|touch\.)?pixiv\.net"
+BASE_PATTERN = r"(?:https?://)?(?:www\.|touch\.)?ph?ixiv\.net"
 USER_PATTERN = BASE_PATTERN + r"/(?:en/)?users/(\d+)"
 
 
@@ -27,8 +27,10 @@ class PixivExtractor(Extractor):
     filename_fmt = "{id}_p{num}.{extension}"
     archive_fmt = "{id}{suffix}.{extension}"
     cookies_domain = ".pixiv.net"
-    sanity_url = "https://s.pximg.net/common/images/limit_sanity_level_360.png"
-    mypixiv_url = "https://s.pximg.net/common/images/limit_mypixiv_360.png"
+    limit_url = "https://s.pximg.net/common/images/limit_"
+    # https://s.pximg.net/common/images/limit_sanity_level_360.png
+    # https://s.pximg.net/common/images/limit_unviewable_360.png
+    # https://s.pximg.net/common/images/limit_mypixiv_360.png
 
     def _init(self):
         self.api = PixivAppAPI(self)
@@ -41,6 +43,10 @@ class PixivExtractor(Extractor):
         self.meta_comments = self.config("comments")
         self.meta_captions = self.config("captions")
 
+        if self.sanity_workaround or self.meta_captions:
+            self.meta_captions_sub = util.re(
+                r'<a href="/jump\.php\?([^"]+)').sub
+
     def items(self):
         tags = self.config("tags", "japanese")
         if tags == "original":
@@ -67,11 +73,14 @@ class PixivExtractor(Extractor):
             files = self._extract_files(work)
 
             if self.meta_user:
-                work.update(self.api.user_detail(work["user"]["id"]))
+                work.update(self.api.user_detail(str(work["user"]["id"])))
             if self.meta_comments:
-                if work["total_comments"]:
-                    work["comments"] = list(
-                        self.api.illust_comments(work["id"]))
+                if work["total_comments"] and not work.get("_ajax"):
+                    try:
+                        work["comments"] = list(
+                            self.api.illust_comments(work["id"]))
+                    except Exception:
+                        work["comments"] = ()
                 else:
                     work["comments"] = ()
             if self.meta_bookmark and work["is_bookmarked"]:
@@ -82,7 +91,8 @@ class PixivExtractor(Extractor):
                     not work.get("_mypixiv") and not work.get("_ajax"):
                 body = self._request_ajax("/illust/" + str(work["id"]))
                 if body:
-                    work["caption"] = text.unescape(body["illustComment"])
+                    work["caption"] = self._sanitize_ajax_caption(
+                        body["illustComment"])
 
             if transform_tags:
                 transform_tags(work)
@@ -110,25 +120,54 @@ class PixivExtractor(Extractor):
             return [
                 {
                     "url"   : img["image_urls"]["original"],
-                    "suffix": "_p{:02}".format(num),
+                    "suffix": f"_p{num:02}",
+                    "_fallback": self._fallback_image(img),
                 }
                 for num, img in enumerate(meta_pages)
             ]
 
         url = meta_single_page["original_image_url"]
-        if url == self.sanity_url:
-            work["_ajax"] = True
-            self.log.warning("%s: 'limit_sanity_level' warning", work["id"])
-            if self.sanity_workaround:
-                body = self._request_ajax("/illust/" + str(work["id"]))
-                return self._extract_ajax(work, body)
+        if url.startswith(self.limit_url):
+            work_id = work["id"]
+            self.log.debug("%s: %s", work_id, url)
 
-        elif url == self.mypixiv_url:
-            work["_mypixiv"] = True
-            self.log.warning("%s: 'My pixiv' locked", work["id"])
+            limit_type = url.rpartition("/")[2]
+            if limit_type in (
+                "limit_",  # for '_extend_sanity()' inserts
+                "limit_unviewable_360.png",
+                "limit_sanity_level_360.png",
+            ):
+                work["_ajax"] = True
+                self.log.warning("%s: 'limit_sanity_level' warning", work_id)
+                if self.sanity_workaround:
+                    body = self._request_ajax("/illust/" + str(work_id))
+                    if work["type"] == "ugoira":
+                        if not self.load_ugoira:
+                            return ()
+                        self.log.info("%s: Retrieving Ugoira AJAX metadata",
+                                      work["id"])
+                        try:
+                            self._extract_ajax(work, body)
+                            return self._extract_ugoira(work, url)
+                        except Exception as exc:
+                            self.log.debug("", exc_info=exc)
+                            self.log.warning(
+                                "%s: Unable to extract Ugoira URL. Provide "
+                                "logged-in cookies to access it", work["id"])
+                    else:
+                        return self._extract_ajax(work, body)
+
+            elif limit_type == "limit_mypixiv_360.png":
+                work["_mypixiv"] = True
+                self.log.warning("%s: 'My pixiv' locked", work_id)
+
+            else:
+                work["_mypixiv"] = True  # stop further processing
+                self.log.error("%s: Unknown 'limit' URL type: %s",
+                               work_id, limit_type)
 
         elif work["type"] != "ugoira":
-            return ({"url": url},)
+            return ({"url": url, "_fallback": self._fallback_image(url)},)
 
         elif self.load_ugoira:
             try:
@@ -141,7 +180,12 @@ class PixivExtractor(Extractor):
         return ()
 
     def _extract_ugoira(self, work, img_url):
-        ugoira = self.api.ugoira_metadata(work["id"])
+        if work.get("_ajax"):
+            ugoira = self._request_ajax(
+                "/illust/" + str(work["id"]) + "/ugoira_meta")
+            img_url = ugoira["src"]
+        else:
+            ugoira = self.api.ugoira_metadata(work["id"])
         work["_ugoira_frame_data"] = work["frames"] = frames = ugoira["frames"]
         work["_ugoira_original"] = self.load_ugoira_original
         work["_http_adjust_extension"] = False
@@ -159,7 +203,7 @@ class PixivExtractor(Extractor):
 
                 for ext in ("jpg", "png", "gif"):
                     try:
-                        url = "{}0.{}".format(base, ext)
+                        url = f"{base}0.{ext}"
                         self.request(url, method="HEAD")
                         break
                     except exception.HttpError:
@@ -170,23 +214,26 @@ class PixivExtractor(Extractor):
 
             return [
                 {
-                    "url": "{}{}.{}".format(base, num, ext),
-                    "suffix": "_p{:02}".format(num),
+                    "url": f"{base}{num}.{ext}",
+                    "suffix": f"_p{num:02}",
                     "_ugoira_frame_index": num,
                 }
                 for num in range(len(frames))
             ]
 
         else:
-            zip_url = ugoira["zip_urls"]["medium"]
+            if work.get("_ajax"):
+                zip_url = ugoira["originalSrc"]
+            else:
+                zip_url = ugoira["zip_urls"]["medium"]
             work["date_url"] = self._date_from_url(zip_url)
             url = zip_url.replace("_ugoira600x600", "_ugoira1920x1080", 1)
             return ({"url": url},)
 
     def _request_ajax(self, endpoint):
-        url = "{}/ajax{}".format(self.root, endpoint)
+        url = f"{self.root}/ajax{endpoint}"
         try:
-            return self.request(url, headers=self.headers_web).json()["body"]
+            return self.request_json(url, headers=self.headers_web)["body"]
         except Exception:
             return None
 
@@ -230,7 +277,7 @@ class PixivExtractor(Extractor):
                 translated_name = None
             tags.append({"name": name, "translated_name": translated_name})
 
-        work["caption"] = text.unescape(body["illustComment"])
+        work["caption"] = self._sanitize_ajax_caption(body["illustComment"])
         work["page_count"] = count = body["pageCount"]
         if count == 1:
             return ({"url": url},)
@@ -238,8 +285,8 @@ class PixivExtractor(Extractor):
         base, _, ext = url.rpartition("_p0.")
         return [
             {
-                "url"   : "{}_p{}.{}".format(base, num, ext),
-                "suffix": "_p{:02}".format(num),
+                "url"   : f"{base}_p{num}.{ext}",
+                "suffix": f"_p{num:02}",
             }
             for num in range(count)
         ]
@@ -263,14 +310,37 @@ class PixivExtractor(Extractor):
 
         for ext in ("jpg", "png", "gif"):
             try:
-                url = "{}_p0.{}".format(base, ext)
+                url = f"{base}_p0.{ext}"
                 self.request(url, method="HEAD")
                 return url
             except exception.HttpError:
                 pass
 
-    @staticmethod
-    def _date_from_url(url, offset=timedelta(hours=9)):
+    def _sanitize_ajax_caption(self, caption):
+        if not caption:
+            return ""
+        return text.unescape(self.meta_captions_sub(
+            lambda m: '<a href="' + text.unquote(m[1]), caption))
+
+    def _fallback_image(self, src):
+        if isinstance(src, str):
+            urls = None
+            orig = src
+        else:
+            urls = src["image_urls"]
+            orig = urls["original"]
+
+        base = orig.rpartition(".")[0]
+        yield base.replace("-original/", "-master/", 1) + "_master1200.jpg"
+
+        if urls is None:
+            return
+
+        for fmt in ("large", "medium", "square_medium"):
+            if fmt in urls:
+                yield urls[fmt]
+
+    def _date_from_url(self, url, offset=timedelta(hours=9)):
         try:
             _, _, _, _, _, y, m, d, H, M, S, _ = url.split("/")
             return datetime(
@@ -278,12 +348,11 @@ class PixivExtractor(Extractor):
         except Exception:
             return None
 
-    @staticmethod
-    def _make_work(kind, url, user):
+    def _make_work(self, kind, url, user):
         p = url.split("/")
         return {
-            "create_date"     : "{}-{}-{}T{}:{}:{}+09:00".format(
-                p[5], p[6], p[7], p[8], p[9], p[10]) if len(p) > 9 else None,
+            "create_date"     : (f"{p[5]}-{p[6]}-{p[7]}T{p[8]}:{p[9]}:{p[10]}"
+                                 f"+09:00" if len(p) > 9 else None),
             "height"          : 0,
             "id"              : kind,
             "image_urls"      : None,
@@ -307,23 +376,15 @@ class PixivExtractor(Extractor):
         return {}
 
 
-class PixivUserExtractor(PixivExtractor):
+class PixivUserExtractor(Dispatch, PixivExtractor):
     """Extractor for a pixiv user profile"""
-    subcategory = "user"
     pattern = (BASE_PATTERN + r"/(?:"
                r"(?:en/)?u(?:sers)?/|member\.php\?id=|(?:mypage\.php)?#id="
                r")(\d+)(?:$|[?#])")
     example = "https://www.pixiv.net/en/users/12345"
 
-    def __init__(self, match):
-        PixivExtractor.__init__(self, match)
-        self.user_id = match.group(1)
-
-    def initialize(self):
-        pass
-
     def items(self):
-        base = "{}/users/{}/".format(self.root, self.user_id)
+        base = f"{self.root}/users/{self.groups[0]}/"
         return self._dispatch_extractors((
             (PixivAvatarExtractor       , base + "avatar"),
             (PixivBackgroundExtractor   , base + "background"),
@@ -331,7 +392,10 @@ class PixivUserExtractor(PixivExtractor):
             (PixivFavoriteExtractor     , base + "bookmarks/artworks"),
             (PixivNovelBookmarkExtractor, base + "bookmarks/novels"),
             (PixivNovelUserExtractor    , base + "novels"),
-        ), ("artworks",))
+        ), ("artworks",), (
+            ("bookmark", "novel-bookmark"),
+            ("user"    , "novel-user"),
+        ))
 
 
 class PixivArtworksExtractor(PixivExtractor):
@@ -374,14 +438,15 @@ class PixivArtworksExtractor(PixivExtractor):
 
         if self.sanity_workaround:
             body = self._request_ajax(
-                "/user/{}/profile/all".format(self.user_id))
+                f"/user/{self.user_id}/profile/all")
             try:
                 ajax_ids = list(map(int, body["illusts"]))
                 ajax_ids.extend(map(int, body["manga"]))
                 ajax_ids.sort()
             except Exception as exc:
-                self.log.warning("Unable to collect artwork IDs using AJAX "
-                                 "API (%s: %s)", exc.__class__.__name__, exc)
+                self.log.warning("u%s: Failed to collect artwork IDs "
+                                 "using AJAX API (%s: %s)",
+                                 self.user_id, exc.__class__.__name__, exc)
             else:
                 works = self._extend_sanity(works, ajax_ids)
 
@@ -410,7 +475,7 @@ class PixivArtworksExtractor(PixivExtractor):
                 elif ajax_id > work_id:
                     index -= 1
                     self.log.debug("Inserting work %s", ajax_id)
-                    yield self._make_work(ajax_id, self.sanity_url, user)
+                    yield self._make_work(ajax_id, self.limit_url, user)
 
                 else:  # ajax_id < work_id
                     break
@@ -420,7 +485,7 @@ class PixivArtworksExtractor(PixivExtractor):
         while index >= 0:
             ajax_id = ajax_ids[index]
             self.log.debug("Inserting work %s", ajax_id)
-            yield self._make_work(ajax_id, self.sanity_url, user)
+            yield self._make_work(ajax_id, self.limit_url, user)
             index -= 1
 
 
@@ -477,22 +542,16 @@ class PixivMeExtractor(PixivExtractor):
     pattern = r"(?:https?://)?pixiv\.me/([^/?#]+)"
     example = "https://pixiv.me/USER"
 
-    def __init__(self, match):
-        PixivExtractor.__init__(self, match)
-        self.account = match.group(1)
-
     def items(self):
-        url = "https://pixiv.me/" + self.account
-        data = {"_extractor": PixivUserExtractor}
-        response = self.request(
-            url, method="HEAD", allow_redirects=False, notfound="user")
-        yield Message.Queue, response.headers["Location"], data
+        url = "https://pixiv.me/" + self.groups[0]
+        location = self.request_location(url, notfound="user")
+        yield Message.Queue, location, {"_extractor": PixivUserExtractor}
 
 
 class PixivWorkExtractor(PixivExtractor):
     """Extractor for a single pixiv work/illustration"""
     subcategory = "work"
-    pattern = (r"(?:https?://)?(?:(?:www\.|touch\.)?pixiv\.net"
+    pattern = (r"(?:https?://)?(?:(?:www\.|touch\.)?ph?ixiv\.net"
                r"/(?:(?:en/)?artworks/"
                r"|member_illust\.php\?(?:[^&]+&)*illust_id=)(\d+)"
                r"|(?:i(?:\d+\.pixiv|\.pximg)\.net"
@@ -502,7 +561,7 @@ class PixivWorkExtractor(PixivExtractor):
 
     def __init__(self, match):
         PixivExtractor.__init__(self, match)
-        self.illust_id = match.group(1) or match.group(2)
+        self.illust_id = match[1] or match[2]
 
     def works(self):
         works = (self.api.illust_detail(self.illust_id),)
@@ -587,7 +646,7 @@ class PixivFavoriteExtractor(PixivExtractor):
         for preview in self.api.user_following(self.user_id, restrict):
             user = preview["user"]
             user["_extractor"] = PixivUserExtractor
-            url = "https://www.pixiv.net/users/{}".format(user["id"])
+            url = f"https://www.pixiv.net/users/{user['id']}"
             yield Message.Queue, url, user
 
 
@@ -602,13 +661,17 @@ class PixivRankingExtractor(PixivExtractor):
 
     def __init__(self, match):
         PixivExtractor.__init__(self, match)
-        self.query = match.group(1)
+        self.query = match[1]
         self.mode = self.date = None
 
     def works(self):
         ranking = self.ranking
-        for ranking["rank"], work in enumerate(
-                self.api.illust_ranking(self.mode, self.date), 1):
+
+        works = self.api.illust_ranking(self.mode, self.date)
+        if self.type:
+            works = filter(lambda work, t=self.type: work["type"] == t, works)
+
+        for ranking["rank"], work in enumerate(works, 1):
             yield work
 
     def metadata(self):
@@ -634,12 +697,12 @@ class PixivRankingExtractor(PixivExtractor):
         try:
             self.mode = mode = mode_map[mode]
         except KeyError:
-            raise exception.StopExtraction("Invalid mode '%s'", mode)
+            raise exception.AbortExtraction(f"Invalid mode '{mode}'")
 
         date = query.get("date")
         if date:
             if len(date) == 8 and date.isdecimal():
-                date = "{}-{}-{}".format(date[0:4], date[4:6], date[6:8])
+                date = f"{date[0:4]}-{date[4:6]}-{date[6:8]}"
             else:
                 self.log.warning("invalid date '%s'", date)
                 date = None
@@ -648,10 +711,13 @@ class PixivRankingExtractor(PixivExtractor):
             date = (now - timedelta(days=1)).strftime("%Y-%m-%d")
         self.date = date
 
+        self.type = type = query.get("content")
+
         self.ranking = ranking = {
             "mode": mode,
             "date": self.date,
             "rank": 0,
+            "type": type or "all",
         }
         return {"ranking": ranking}
 
@@ -684,7 +750,7 @@ class PixivSearchExtractor(PixivExtractor):
             try:
                 self.word = query["word"]
             except KeyError:
-                raise exception.StopExtraction("Missing search term")
+                raise exception.AbortExtraction("Missing search term")
 
         sort = query.get("order", "date_d")
         sort_map = {
@@ -697,7 +763,7 @@ class PixivSearchExtractor(PixivExtractor):
         try:
             self.sort = sort = sort_map[sort]
         except KeyError:
-            raise exception.StopExtraction("Invalid search order '%s'", sort)
+            raise exception.AbortExtraction(f"Invalid search order '{sort}'")
 
         target = query.get("s_mode", "s_tag_full")
         target_map = {
@@ -708,7 +774,7 @@ class PixivSearchExtractor(PixivExtractor):
         try:
             self.target = target = target_map[target]
         except KeyError:
-            raise exception.StopExtraction("Invalid search mode '%s'", target)
+            raise exception.AbortExtraction(f"Invalid search mode '{target}'")
 
         self.date_start = query.get("scd")
         self.date_end = query.get("ecd")
@@ -749,7 +815,7 @@ class PixivPixivisionExtractor(PixivExtractor):
 
     def __init__(self, match):
         PixivExtractor.__init__(self, match)
-        self.pixivision_id = match.group(1)
+        self.pixivision_id = match[1]
 
     def works(self):
         return (
@@ -798,18 +864,71 @@ class PixivSeriesExtractor(PixivExtractor):
             yield work
 
 
-class PixivNovelExtractor(PixivExtractor):
-    """Extractor for pixiv novels"""
-    subcategory = "novel"
-    request_interval = (0.5, 1.5)
-    pattern = BASE_PATTERN + r"/n(?:ovel/show\.php\?id=|/)(\d+)"
-    example = "https://www.pixiv.net/novel/show.php?id=12345"
-
-    def __init__(self, match):
-        PixivExtractor.__init__(self, match)
-        self.novel_id = match.group(1)
+class PixivSketchExtractor(Extractor):
+    """Extractor for user pages on sketch.pixiv.net"""
+    category = "pixiv"
+    subcategory = "sketch"
+    directory_fmt = ("{category}", "sketch", "{user[unique_name]}")
+    filename_fmt = "{post_id} {id}.{extension}"
+    archive_fmt = "S{user[id]}_{id}"
+    root = "https://sketch.pixiv.net"
+    cookies_domain = ".pixiv.net"
+    pattern = r"(?:https?://)?sketch\.pixiv\.net/@([^/?#]+)"
+    example = "https://sketch.pixiv.net/@USER"
 
     def items(self):
+        self.username = self.groups[0]
+        headers = {"Referer": f"{self.root}/@{self.username}"}
+
+        for post in self.posts():
+            media = post["media"]
+            post["post_id"] = post["id"]
+            post["date"] = text.parse_datetime(
+                post["created_at"], "%Y-%m-%dT%H:%M:%S.%f%z")
+            util.delete_items(post, ("id", "media", "_links"))
+
+            yield Message.Directory, post
+            post["_http_headers"] = headers
+
+            for photo in media:
+                original = photo["photo"]["original"]
+                post["id"] = photo["id"]
+                post["width"] = original["width"]
+                post["height"] = original["height"]
+
+                url = original["url"]
+                text.nameext_from_url(url, post)
+                yield Message.Url, url, post
+
+    def posts(self):
+        url = f"{self.root}/api/walls/@{self.username}/posts/public.json"
+        headers = {
+            "Accept": "application/vnd.sketch-v4+json",
+            "Referer": self.root + "/",
+            "X-Requested-With": f"{self.root}/@{self.username}",
+        }
+
+        while True:
+            data = self.request_json(url, headers=headers)
+            yield from data["data"]["items"]
+
+            next_url = data["_links"].get("next")
+            if not next_url:
+                return
+            url = self.root + next_url["href"]
+
+
+###############################################################################
+# Novels ######################################################################
+
+class PixivNovelExtractor(PixivExtractor):
+    """Base class for pixiv novel extractors"""
+    category = "pixiv-novel"
+    request_interval = (0.5, 1.5)
+
+    def items(self):
+        self.novel_id = self.groups[0]
+
         tags = self.config("tags", "japanese")
         if tags == "original":
             transform_tags = None
@@ -826,22 +945,12 @@ class PixivNovelExtractor(PixivExtractor):
         embeds = self.config("embeds")
         covers = self.config("covers")
 
-        if embeds:
-            headers = {
-                "User-Agent"    : "Mozilla/5.0",
-                "App-OS"        : None,
-                "App-OS-Version": None,
-                "App-Version"   : None,
-                "Referer"       : self.root + "/",
-                "Authorization" : None,
-            }
-
         novels = self.novels()
         if self.max_posts:
             novels = itertools.islice(novels, self.max_posts)
         for novel in novels:
             if self.meta_user:
-                novel.update(self.api.user_detail(novel["user"]["id"]))
+                novel.update(self.api.user_detail(str(novel["user"]["id"])))
             if self.meta_comments:
                 if novel["total_comments"]:
                     novel["comments"] = list(
@@ -876,7 +985,7 @@ class PixivNovelExtractor(PixivExtractor):
                        path.rpartition(".")[0].replace("_master1200", ""))
                 novel["date_url"] = self._date_from_url(url)
                 novel["num"] += 1
-                novel["suffix"] = "_p{:02}".format(novel["num"])
+                novel["suffix"] = f"_p{novel['num']:02}"
                 novel["_fallback"] = (url + ".png",)
                 url_jpg = url + ".jpg"
                 text.nameext_from_url(url_jpg, novel)
@@ -894,20 +1003,21 @@ class PixivNovelExtractor(PixivExtractor):
                         illusts[marker[11:].partition("-")[0]] = None
 
                 if desktop:
-                    novel_id = str(novel["id"])
-                    url = "{}/novel/show.php?id={}".format(
-                        self.root, novel_id)
-                    data = util.json_loads(text.extr(
-                        self.request(url, headers=headers).text,
-                        "id=\"meta-preload-data\" content='", "'"))
+                    try:
+                        body = self._request_ajax("/novel/" + str(novel["id"]))
+                        images = body["textEmbeddedImages"].values()
+                    except Exception as exc:
+                        self.log.warning(
+                            "%s: Failed to get embedded novel images (%s: %s)",
+                            novel["id"], exc.__class__.__name__, exc)
+                        images = ()
 
-                    for image in (data["novel"][novel_id]
-                                  ["textEmbeddedImages"]).values():
+                    for image in images:
                         url = image.pop("urls")["original"]
                         novel.update(image)
                         novel["date_url"] = self._date_from_url(url)
                         novel["num"] += 1
-                        novel["suffix"] = "_p{:02}".format(novel["num"])
+                        novel["suffix"] = f"_p{novel['num']:02}"
                         text.nameext_from_url(url, novel)
                         yield Message.Url, url, novel
 
@@ -916,10 +1026,17 @@ class PixivNovelExtractor(PixivExtractor):
                     novel["date_url"] = None
                     for illust_id in illusts:
                         novel["num"] += 1
-                        novel["suffix"] = "_p{:02}".format(novel["num"])
-                        url = "{}/artworks/{}".format(self.root, illust_id)
+                        novel["suffix"] = f"_p{novel['num']:02}"
+                        url = f"{self.root}/artworks/{illust_id}"
                         yield Message.Queue, url, novel
 
+
+class PixivNovelNovelExtractor(PixivNovelExtractor):
+    """Extractor for pixiv novels"""
+    subcategory = "novel"
+    pattern = BASE_PATTERN + r"/n(?:ovel/show\.php\?id=|/)(\d+)"
+    example = "https://www.pixiv.net/novel/show.php?id=12345"
+
     def novels(self):
         novel = self.api.novel_detail(self.novel_id)
         if self.config("full-series") and novel["series"]:
@@ -930,7 +1047,7 @@ class PixivNovelExtractor(PixivExtractor):
 
 class PixivNovelUserExtractor(PixivNovelExtractor):
     """Extractor for pixiv users' novels"""
-    subcategory = "novel-user"
+    subcategory = "user"
     pattern = USER_PATTERN + r"/novels"
     example = "https://www.pixiv.net/en/users/12345/novels"
 
@@ -940,7 +1057,7 @@ class PixivNovelUserExtractor(PixivNovelExtractor):
 
 class PixivNovelSeriesExtractor(PixivNovelExtractor):
     """Extractor for pixiv novel series"""
-    subcategory = "novel-series"
+    subcategory = "series"
     pattern = BASE_PATTERN + r"/novel/series/(\d+)"
     example = "https://www.pixiv.net/novel/series/12345"
 
@@ -950,86 +1067,25 @@ class PixivNovelSeriesExtractor(PixivNovelExtractor):
 
 class PixivNovelBookmarkExtractor(PixivNovelExtractor):
     """Extractor for bookmarked pixiv novels"""
-    subcategory = "novel-bookmark"
+    subcategory = "bookmark"
     pattern = (USER_PATTERN + r"/bookmarks/novels"
                r"(?:/([^/?#]+))?(?:/?\?([^#]+))?")
     example = "https://www.pixiv.net/en/users/12345/bookmarks/novels"
 
-    def __init__(self, match):
-        PixivNovelExtractor.__init__(self, match)
-        self.user_id, self.tag, self.query = match.groups()
-
     def novels(self):
-        if self.tag:
-            tag = text.unquote(self.tag)
-        else:
-            tag = None
+        user_id, tag, query = self.groups
+        tag = text.unquote(tag) if tag else None
 
-        if text.parse_query(self.query).get("rest") == "hide":
+        if text.parse_query(query).get("rest") == "hide":
             restrict = "private"
         else:
             restrict = "public"
 
-        return self.api.user_bookmarks_novel(self.user_id, tag, restrict)
+        return self.api.user_bookmarks_novel(user_id, tag, restrict)
 
 
-class PixivSketchExtractor(Extractor):
-    """Extractor for user pages on sketch.pixiv.net"""
-    category = "pixiv"
-    subcategory = "sketch"
-    directory_fmt = ("{category}", "sketch", "{user[unique_name]}")
-    filename_fmt = "{post_id} {id}.{extension}"
-    archive_fmt = "S{user[id]}_{id}"
-    root = "https://sketch.pixiv.net"
-    cookies_domain = ".pixiv.net"
-    pattern = r"(?:https?://)?sketch\.pixiv\.net/@([^/?#]+)"
-    example = "https://sketch.pixiv.net/@USER"
-
-    def __init__(self, match):
-        Extractor.__init__(self, match)
-        self.username = match.group(1)
-
-    def items(self):
-        headers = {"Referer": "{}/@{}".format(self.root, self.username)}
-
-        for post in self.posts():
-            media = post["media"]
-            post["post_id"] = post["id"]
-            post["date"] = text.parse_datetime(
-                post["created_at"], "%Y-%m-%dT%H:%M:%S.%f%z")
-            util.delete_items(post, ("id", "media", "_links"))
-
-            yield Message.Directory, post
-            post["_http_headers"] = headers
-
-            for photo in media:
-                original = photo["photo"]["original"]
-                post["id"] = photo["id"]
-                post["width"] = original["width"]
-                post["height"] = original["height"]
-
-                url = original["url"]
-                text.nameext_from_url(url, post)
-                yield Message.Url, url, post
-
-    def posts(self):
-        url = "{}/api/walls/@{}/posts/public.json".format(
-            self.root, self.username)
-        headers = {
-            "Accept": "application/vnd.sketch-v4+json",
-            "X-Requested-With": "{}/@{}".format(self.root, self.username),
-            "Referer": self.root + "/",
-        }
-
-        while True:
-            data = self.request(url, headers=headers).json()
-            yield from data["data"]["items"]
-
-            next_url = data["_links"].get("next")
-            if not next_url:
-                return
-            url = self.root + next_url["href"]
-
+###############################################################################
+# API #########################################################################
 
 class PixivAppAPI():
     """Minimal interface for the Pixiv App API for mobile devices
@@ -1235,7 +1291,7 @@ class PixivAppAPI():
                 self.extractor.wait(seconds=300)
                 continue
 
-            raise exception.StopExtraction("API request failed: %s", error)
+            raise exception.AbortExtraction(f"API request failed: {error}")
 
     def _pagination(self, endpoint, params,
                     key_items="illusts", key_data=None):
