@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2017-2023 Mike Fährmann
+# Copyright 2017-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -31,6 +31,7 @@ class RedditExtractor(Extractor):
         parentdir = self.config("parent-directory")
         max_depth = self.config("recursion", 0)
         previews = self.config("previews", True)
+        embeds = self.config("embeds", True)
 
         videos = self.config("videos", True)
         if videos:
@@ -40,6 +41,11 @@ class RedditExtractor(Extractor):
                 self._extract_video = self._extract_video_dash
             videos = True
 
+        selftext = self.config("selftext")
+        if selftext is None:
+            selftext = self.api.comments
+        selftext = True if selftext else False
+
         submissions = self.submissions()
         visited = set()
         depth = 0
@@ -91,16 +97,16 @@ class RedditExtractor(Extractor):
                 elif parentdir:
                     yield Message.Directory, comments[0]
 
+                if selftext and submission:
+                    for url in text.extract_iter(
+                            submission["selftext_html"] or "", ' href="', '"'):
+                        urls.append((url, submission))
+
                 if self.api.comments:
-                    if submission:
-                        for url in text.extract_iter(
-                                submission["selftext_html"] or "",
-                                ' href="', '"'):
-                            urls.append((url, submission))
                     for comment in comments:
                         html = comment["body_html"] or ""
                         href = (' href="' in html)
-                        media = ("media_metadata" in comment)
+                        media = (embeds and "media_metadata" in comment)
 
                         if media or href:
                             comment["date"] = text.parse_timestamp(
@@ -135,7 +141,7 @@ class RedditExtractor(Extractor):
 
                     match = match_submission(url)
                     if match:
-                        extra.append(match.group(1))
+                        extra.append(match[1])
                     elif not match_user(url) and not match_subreddit(url):
                         if previews and "comment" not in data and \
                                 "preview" in data:
@@ -211,8 +217,9 @@ class RedditExtractor(Extractor):
     def _extract_video_dash(self, submission):
         submission["_ytdl_extra"] = {"title": submission["title"]}
         try:
-            return (submission["secure_media"]["reddit_video"]["dash_url"] +
-                    "#__youtubedl_smuggle=%7B%22to_generic%22%3A+1%7D")
+            url = submission["secure_media"]["reddit_video"]["dash_url"]
+            submission["_ytdl_manifest"] = "dash"
+            return url
         except Exception:
             return submission["url"]
 
@@ -257,6 +264,8 @@ class RedditSubredditExtractor(RedditExtractor):
         self.subreddit, sub, params = match.groups()
         self.params = text.parse_query(params)
         if sub:
+            if sub == "search" and "restrict_sr" not in self.params:
+                self.params["restrict_sr"] = "1"
             self.subcategory += "-" + sub
         RedditExtractor.__init__(self, match)
 
@@ -300,7 +309,7 @@ class RedditSubmissionExtractor(RedditExtractor):
 
     def __init__(self, match):
         RedditExtractor.__init__(self, match)
-        self.submission_id = match.group(1)
+        self.submission_id = match[1]
 
     def submissions(self):
         return (self.api.submission(self.submission_id),)
@@ -317,17 +326,17 @@ class RedditImageExtractor(Extractor):
 
     def __init__(self, match):
         Extractor.__init__(self, match)
-        domain = match.group(1)
-        self.path = match.group(2)
+        domain = match[1]
+        self.path = match[2]
         if domain == "preview.redd.it":
             self.domain = "i.redd.it"
             self.query = ""
         else:
             self.domain = domain
-            self.query = match.group(3) or ""
+            self.query = match[3] or ""
 
     def items(self):
-        url = "https://{}/{}{}".format(self.domain, self.path, self.query)
+        url = f"https://{self.domain}/{self.path}{self.query}"
         data = text.nameext_from_url(url)
         yield Message.Directory, data
         yield Message.Url, url, data
@@ -338,22 +347,18 @@ class RedditRedirectExtractor(Extractor):
     category = "reddit"
     subcategory = "redirect"
     pattern = (r"(?:https?://)?(?:"
-               r"(?:\w+\.)?reddit\.com/(?:(?:r)/([^/?#]+)))"
+               r"(?:\w+\.)?reddit\.com/(?:(r|u|user)/([^/?#]+)))"
                r"/s/([a-zA-Z0-9]{10})")
     example = "https://www.reddit.com/r/SUBREDDIT/s/abc456GHIJ"
 
-    def __init__(self, match):
-        Extractor.__init__(self, match)
-        self.subreddit = match.group(1)
-        self.share_url = match.group(2)
-
     def items(self):
-        url = "https://www.reddit.com/r/" + self.subreddit + "/s/" + \
-              self.share_url
+        sub_type, subreddit, share_url = self.groups
+        if sub_type == "u":
+            sub_type = "user"
+        url = f"https://www.reddit.com/{sub_type}/{subreddit}/s/{share_url}"
+        location = self.request_location(url, notfound="submission")
         data = {"_extractor": RedditSubmissionExtractor}
-        response = self.request(url, method="HEAD", allow_redirects=False,
-                                notfound="submission")
-        yield Message.Queue, response.headers["Location"], data
+        yield Message.Queue, location, data
 
 
 class RedditAPI():
@@ -472,8 +477,8 @@ class RedditAPI():
 
         if response.status_code != 200:
             self.log.debug("Server response: %s", data)
-            raise exception.AuthenticationError('"{}: {}"'.format(
-                data.get("error"), data.get("message")))
+            raise exception.AuthenticationError(
+                f"\"{data.get('error')}: {data.get('message')}\"")
         return "Bearer " + data["access_token"]
 
     def _call(self, endpoint, params):
@@ -502,7 +507,8 @@ class RedditAPI():
             try:
                 data = response.json()
             except ValueError:
-                raise exception.StopExtraction(text.remove_html(response.text))
+                raise exception.AbortExtraction(
+                    text.remove_html(response.text))
 
             if "error" in data:
                 if data["error"] == 403:
@@ -510,7 +516,7 @@ class RedditAPI():
                 if data["error"] == 404:
                     raise exception.NotFoundError()
                 self.log.debug(data)
-                raise exception.StopExtraction(data.get("message"))
+                raise exception.AbortExtraction(data.get("message"))
             return data
 
     def _pagination(self, endpoint, params):
@@ -567,8 +573,7 @@ class RedditAPI():
         sid = self.extractor.config(key)
         return self._decode(sid.rpartition("_")[2].lower()) if sid else default
 
-    @staticmethod
-    def _decode(sid):
+    def _decode(self, sid):
         return util.bdecode(sid, "0123456789abcdefghijklmnopqrstuvwxyz")
 
 
