@@ -36,7 +36,7 @@ class MtimePP(PostProcessor):
         if mtime is None:
             return
 
-        pathfmt.kwdict["_mtime"] = (
+        pathfmt.kwdict["_mtime_meta"] = (
             util.datetime_to_timestamp(mtime)
             if isinstance(mtime, datetime) else
             text.parse_int(mtime)
