@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2018-2023 Mike Fährmann
+# Copyright 2018-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,7 +9,7 @@
 """Convert Pixiv Ugoira to WebM"""
 
 from .common import PostProcessor
-from .. import util
+from .. import util, output
 import subprocess
 import tempfile
 import zipfile
@@ -111,16 +111,13 @@ class UgoiraPP(PostProcessor):
             return
 
         self._frames = pathfmt.kwdict["_ugoira_frame_data"]
-        if pathfmt.extension == "zip":
+        index = pathfmt.kwdict.get("_ugoira_frame_index")
+        if index is None:
             self._convert_zip = True
             if self.delete:
                 pathfmt.set_extension(self.extension)
                 pathfmt.build_path()
         else:
-            index = pathfmt.kwdict.get("_ugoira_frame_index")
-            if index is None:
-                return
-
             pathfmt.build_path()
             frame = self._frames[index].copy()
             frame["index"] = index
@@ -138,6 +135,7 @@ class UgoiraPP(PostProcessor):
         if not self._convert_zip:
             return
         self._zip_source = True
+        self._zip_ext = ext = pathfmt.extension
 
         with self._tempdir() as tempdir:
             if tempdir:
@@ -158,9 +156,9 @@ class UgoiraPP(PostProcessor):
             if self.convert(pathfmt, tempdir):
                 if self.delete:
                     pathfmt.delete = True
-                elif pathfmt.extension != "zip":
+                elif pathfmt.extension != ext:
                     self.log.info(pathfmt.filename)
-                    pathfmt.set_extension("zip")
+                    pathfmt.set_extension(ext)
                     pathfmt.build_path()
 
     def convert_from_files(self, pathfmt):
@@ -172,8 +170,8 @@ class UgoiraPP(PostProcessor):
             for frame in self._files:
 
                 # update frame filename extension
-                frame["file"] = name = "{}.{}".format(
-                    frame["file"].partition(".")[0], frame["ext"])
+                frame["file"] = name = \
+                    f"{frame['file'].partition('.')[0]}.{frame['ext']}"
 
                 if tempdir:
                     # move frame into tempdir
@@ -226,21 +224,19 @@ class UgoiraPP(PostProcessor):
             if self._finalize:
                 self._finalize(pathfmt, tempdir)
         except OSError as exc:
-            print()
+            output.stderr_write("\n")
             self.log.error("Unable to invoke FFmpeg (%s: %s)",
                            exc.__class__.__name__, exc)
             self.log.debug("", exc_info=exc)
             pathfmt.realpath = pathfmt.temppath
         except Exception as exc:
-            print()
+            output.stderr_write("\n")
             self.log.error("%s: %s", exc.__class__.__name__, exc)
             self.log.debug("", exc_info=exc)
             pathfmt.realpath = pathfmt.temppath
         else:
             if self.mtime:
-                mtime = pathfmt.kwdict.get("_mtime")
-                if mtime:
-                    util.set_mtime(pathfmt.realpath, mtime)
+                pathfmt.set_mtime()
             return True
 
     def convert_to_archive(self, pathfmt, tempdir):
@@ -257,9 +253,15 @@ class UgoiraPP(PostProcessor):
             ]).encode()
 
         if self._zip_source:
-            self.delete = False
+            zpath = pathfmt.temppath
+            if self.delete:
+                self.delete = False
+            elif self._zip_ext != self.extension:
+                self._copy_file(zpath, pathfmt.realpath)
+                zpath = pathfmt.realpath
+
             if self.metadata:
-                with zipfile.ZipFile(pathfmt.temppath, "a") as zfile:
+                with zipfile.ZipFile(zpath, "a") as zfile:
                     zinfo = zipfile.ZipInfo(metaname)
                     if self.mtime:
                         zinfo.date_time = zfile.infolist()[0].date_time
@@ -296,7 +298,7 @@ class UgoiraPP(PostProcessor):
         out = None if self.output else subprocess.DEVNULL
         retcode = util.Popen(args, stdout=out, stderr=out).wait()
         if retcode:
-            print()
+            output.stderr_write("\n")
             self.log.error("Non-zero exit status when running %s (%s)",
                            args, retcode)
             raise ValueError()
@@ -330,7 +332,7 @@ class UgoiraPP(PostProcessor):
                 last_copy = last.copy()
                 frames.append(last_copy)
                 name, _, ext = last_copy["file"].rpartition(".")
-                last_copy["file"] = "{:>06}.{}".format(int(name)+1, ext)
+                last_copy["file"] = f"{int(name) + 1:>06}.{ext}"
                 shutil.copyfile(tempdir + last["file"],
                                 tempdir + last_copy["file"])
 
@@ -345,10 +347,8 @@ class UgoiraPP(PostProcessor):
             "-f", "image2",
             "-ts_from_file", "2",
             "-pattern_type", "sequence",
-            "-i", "{}%06d.{}".format(
-                tempdir.replace("%", "%%"),
-                frame["file"].rpartition(".")[2]
-            ),
+            "-i", (f"{tempdir.replace('%', '%%')}%06d."
+                   f"{frame['file'].rpartition('.')[2]}"),
         ]
 
     def _process_mkvmerge(self, pathfmt, tempdir):
@@ -359,10 +359,8 @@ class UgoiraPP(PostProcessor):
             self.ffmpeg,
             "-f", "image2",
             "-pattern_type", "sequence",
-            "-i", "{}/%06d.{}".format(
-                tempdir.replace("%", "%%"),
-                self._frames[0]["file"].rpartition(".")[2]
-            ),
+            "-i", (f"{tempdir.replace('%', '%%')}/%06d."
+                   f"{self._frames[0]['file'].rpartition('.')[2]}"),
         ]
 
     def _finalize_mkvmerge(self, pathfmt, tempdir):
@@ -380,14 +378,13 @@ class UgoiraPP(PostProcessor):
 
     def _write_ffmpeg_concat(self, tempdir):
         content = ["ffconcat version 1.0"]
-        append = content.append
 
         for frame in self._frames:
-            append("file '{}'\nduration {}".format(
-                frame["file"], frame["delay"] / 1000))
+            content.append(f"file '{frame['file']}'\n"
+                           f"duration {frame['delay'] / 1000}")
         if self.repeat:
-            append("file '{}'".format(frame["file"]))
-        append("")
+            content.append(f"file '{frame['file']}'")
+        content.append("")
 
         ffconcat = tempdir + "/ffconcat.txt"
         with open(ffconcat, "w") as fp:
@@ -396,14 +393,13 @@ class UgoiraPP(PostProcessor):
 
     def _write_mkvmerge_timecodes(self, tempdir):
         content = ["# timecode format v2"]
-        append = content.append
 
         delay_sum = 0
         for frame in self._frames:
-            append(str(delay_sum))
+            content.append(str(delay_sum))
             delay_sum += frame["delay"]
-        append(str(delay_sum))
-        append("")
+        content.append(str(delay_sum))
+        content.append("")
 
         timecodes = tempdir + "/timecodes.tc"
         with open(timecodes, "w") as fp:
@@ -412,24 +408,22 @@ class UgoiraPP(PostProcessor):
 
     def calculate_framerate(self, frames):
         if self._delay_is_uniform(frames):
-            return ("1000/{}".format(frames[0]["delay"]), None)
+            return (f"1000/{frames[0]['delay']}", None)
 
         if not self.uniform:
             gcd = self._delay_gcd(frames)
             if gcd >= 10:
-                return (None, "1000/{}".format(gcd))
+                return (None, f"1000/{gcd}")
 
         return (None, None)
 
-    @staticmethod
-    def _delay_gcd(frames):
+    def _delay_gcd(self, frames):
         result = frames[0]["delay"]
         for f in frames:
             result = gcd(result, f["delay"])
         return result
 
-    @staticmethod
-    def _delay_is_uniform(frames):
+    def _delay_is_uniform(self, frames):
         delay = frames[0]["delay"]
         for f in frames:
             if f["delay"] != delay:
