@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2021-2023 Mike Fährmann
+# Copyright 2021-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,7 +8,6 @@
 
 """Helpers for interacting with youtube-dl"""
 
-import re
 import shlex
 import itertools
 from . import text, util, exception
@@ -20,7 +19,7 @@ def import_module(module_name):
             return __import__("yt_dlp")
         except (ImportError, SyntaxError):
             return __import__("youtube_dl")
-    return __import__(module_name.replace("-", "_"))
+    return util.import_file(module_name)
 
 
 def construct_YoutubeDL(module, obj, user_opts, system_opts=None):
@@ -42,7 +41,7 @@ def construct_YoutubeDL(module, obj, user_opts, system_opts=None):
     try:
         opts = parse_command_line(module, argv) if argv else user_opts
     except SystemExit:
-        raise exception.StopExtraction("Invalid command-line option")
+        raise exception.AbortExtraction("Invalid command-line option")
 
     if opts.get("format") is None:
         opts["format"] = config("format")
@@ -50,12 +49,20 @@ def construct_YoutubeDL(module, obj, user_opts, system_opts=None):
         opts["nopart"] = not config("part", True)
     if opts.get("updatetime") is None:
         opts["updatetime"] = config("mtime", True)
-    if opts.get("ratelimit") is None:
-        opts["ratelimit"] = text.parse_bytes(config("rate"), None)
     if opts.get("min_filesize") is None:
         opts["min_filesize"] = text.parse_bytes(config("filesize-min"), None)
     if opts.get("max_filesize") is None:
         opts["max_filesize"] = text.parse_bytes(config("filesize-max"), None)
+    if opts.get("ratelimit") is None:
+        rate = config("rate")
+        if rate:
+            func = util.build_selection_func(rate, 0, text.parse_bytes)
+            if hasattr(func, "args"):
+                opts["__gdl_ratelimit_func"] = func
+            else:
+                opts["ratelimit"] = func() or None
+        else:
+            opts["ratelimit"] = None
 
     raw_opts = config("raw-options")
     if raw_opts:
@@ -65,13 +72,14 @@ def construct_YoutubeDL(module, obj, user_opts, system_opts=None):
     if system_opts:
         opts.update(system_opts)
 
+    opts["__gdl_initialize"] = True
     return module.YoutubeDL(opts)
 
 
 def parse_command_line(module, argv):
     parser, opts, args = module.parseOpts(argv)
 
-    ytdlp = (module.__name__ == "yt_dlp")
+    ytdlp = hasattr(module, "cookies")
     std_headers = module.std_headers
 
     try:
@@ -141,7 +149,7 @@ def parse_command_line(module, argv):
         if name not in compat_opts:
             return False
         compat_opts.discard(name)
-        compat_opts.update(["*%s" % name])
+        compat_opts.update([f"*{name}"])
         return True
 
     def set_default_compat(
@@ -206,7 +214,7 @@ def parse_command_line(module, argv):
                 if "pre_process" not in parse_metadata:
                     parse_metadata["pre_process"] = []
                 parse_metadata["pre_process"].append(
-                    "title:%s" % opts.metafromtitle)
+                    f"title:{opts.metafromtitle}")
             opts.parse_metadata = {
                 k: list(itertools.chain.from_iterable(map(
                         metadataparser_actions, v)))
@@ -216,7 +224,7 @@ def parse_command_line(module, argv):
             if parse_metadata is None:
                 parse_metadata = []
             if opts.metafromtitle is not None:
-                parse_metadata.append("title:%s" % opts.metafromtitle)
+                parse_metadata.append(f"title:{opts.metafromtitle}")
             opts.parse_metadata = list(itertools.chain.from_iterable(map(
                 metadataparser_actions, parse_metadata)))
 
@@ -252,13 +260,12 @@ def parse_command_line(module, argv):
 
     cookiesfrombrowser = getattr(opts, "cookiesfrombrowser", None)
     if cookiesfrombrowser:
-        match = re.fullmatch(r"""(?x)
+        pattern = util.re(r"""(?x)
             (?P<name>[^+:]+)
             (?:\s*\+\s*(?P<keyring>[^:]+))?
             (?:\s*:\s*(?!:)(?P<profile>.+?))?
-            (?:\s*::\s*(?P<container>.+))?
-        """, cookiesfrombrowser)
-        if match:
+            (?:\s*::\s*(?P<container>.+))?""")
+        if match := pattern.fullmatch(cookiesfrombrowser):
             browser, keyring, profile, container = match.groups()
             if keyring is not None:
                 keyring = keyring.upper()
@@ -518,7 +525,7 @@ def legacy_postprocessors(opts, module, ytdlp, compat_opts):
             if len(dur) == 2 and all(t is not None for t in dur):
                 remove_ranges.append(tuple(dur))
                 continue
-        remove_chapters_patterns.append(re.compile(regex))
+        remove_chapters_patterns.append(util.re(regex))
     if opts.remove_chapters or sponsorblock_query:
         postprocessors.append({
             "key": "ModifyChapters",
