@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2016-2023 Mike Fährmann
+# Copyright 2016-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,7 +9,7 @@
 """Extractors for https://www.imagefap.com/"""
 
 from .common import Extractor, Message
-from .. import text, util, exception
+from .. import text, exception
 
 BASE_PATTERN = r"(?:https?://)?(?:www\.|beta\.)?imagefap\.com"
 
@@ -31,7 +31,7 @@ class ImagefapExtractor(Extractor):
             msg = text.extr(response.text, '<div class="mt-4', '<')
             if msg:
                 msg = " ".join(msg.partition(">")[2].split())
-                raise exception.StopExtraction("'%s'", msg)
+                raise exception.AbortExtraction(f"'{msg}'")
             self.log.warning("HTTP redirect to %s", response.url)
 
         return response
@@ -45,11 +45,11 @@ class ImagefapGalleryExtractor(ImagefapExtractor):
 
     def __init__(self, match):
         ImagefapExtractor.__init__(self, match)
-        self.gid = match.group(1)
+        self.gid = match[1]
         self.image_id = ""
 
     def items(self):
-        url = "{}/gallery/{}".format(self.root, self.gid)
+        url = f"{self.root}/gallery/{self.gid}"
         page = self.request(url).text
         data = self.get_job_metadata(page)
         yield Message.Directory, data
@@ -81,12 +81,12 @@ class ImagefapGalleryExtractor(ImagefapExtractor):
 
     def get_images(self):
         """Collect image-urls and -metadata"""
-        url = "{}/photo/{}/".format(self.root, self.image_id)
+        url = f"{self.root}/photo/{self.image_id}/"
         params = {"gid": self.gid, "idx": 0, "partial": "true"}
         headers = {
             "Content-Type": "application/x-www-form-urlencoded",
             "X-Requested-With": "XMLHttpRequest",
-            "Referer": "{}?pgid=&gid={}&page=0".format(url, self.image_id)
+            "Referer": f"{url}?pgid=&gid={self.image_id}&page=0"
         }
 
         num = 0
@@ -116,7 +116,7 @@ class ImagefapImageExtractor(ImagefapExtractor):
 
     def __init__(self, match):
         ImagefapExtractor.__init__(self, match)
-        self.image_id = match.group(1)
+        self.image_id = match[1]
 
     def items(self):
         url, data = self.get_image()
@@ -124,18 +124,16 @@ class ImagefapImageExtractor(ImagefapExtractor):
         yield Message.Url, url, data
 
     def get_image(self):
-        url = "{}/photo/{}/".format(self.root, self.image_id)
+        url = f"{self.root}/photo/{self.image_id}/"
         page = self.request(url).text
 
         url, pos = text.extract(
             page, 'original="', '"')
-        info, pos = text.extract(
-            page, '<script type="application/ld+json">', '</script>', pos)
         image_id, pos = text.extract(
             page, 'id="imageid_input" value="', '"', pos)
         gallery_id, pos = text.extract(
             page, 'id="galleryid_input" value="', '"', pos)
-        info = util.json_loads(info)
+        info = self._extract_jsonld(page)
 
         return url, text.nameext_from_url(url, {
             "title": text.unescape(info["name"]),
@@ -163,7 +161,7 @@ class ImagefapFolderExtractor(ImagefapExtractor):
 
     def items(self):
         for gallery_id, name, folder in self.galleries(self.folder_id):
-            url = "{}/gallery/{}".format(self.root, gallery_id)
+            url = f"{self.root}/gallery/{gallery_id}"
             data = {
                 "gallery_id": gallery_id,
                 "title"     : text.unescape(name),
@@ -177,14 +175,13 @@ class ImagefapFolderExtractor(ImagefapExtractor):
         if folder_id == "-1":
             folder_name = "Uncategorized"
             if self._id:
-                url = "{}/usergallery.php?userid={}&folderid=-1".format(
-                    self.root, self.user)
+                url = (f"{self.root}/usergallery.php"
+                       f"?userid={self.user}&folderid=-1")
             else:
-                url = "{}/profile/{}/galleries?folderid=-1".format(
-                    self.root, self.user)
+                url = f"{self.root}/profile/{self.user}/galleries?folderid=-1"
         else:
             folder_name = None
-            url = "{}/organizer/{}/".format(self.root, folder_id)
+            url = f"{self.root}/organizer/{folder_id}/"
 
         params = {"page": 0}
         extr = text.extract_from(self.request(url, params=params).text)
@@ -224,19 +221,17 @@ class ImagefapUserExtractor(ImagefapExtractor):
 
         for folder_id in self.folders():
             if folder_id == "-1":
-                url = "{}/profile/{}/galleries?folderid=-1".format(
-                    self.root, self.user)
+                url = f"{self.root}/profile/{self.user}/galleries?folderid=-1"
             else:
-                url = "{}/organizer/{}/".format(self.root, folder_id)
+                url = f"{self.root}/organizer/{folder_id}/"
             yield Message.Queue, url, data
 
     def folders(self):
         """Return a list of folder IDs of a user"""
         if self.user:
-            url = "{}/profile/{}/galleries".format(self.root, self.user)
+            url = f"{self.root}/profile/{self.user}/galleries"
         else:
-            url = "{}/usergallery.php?userid={}".format(
-                self.root, self.user_id)
+            url = f"{self.root}/usergallery.php?userid={self.user_id}"
 
         response = self.request(url)
         self.user = response.url.split("/")[-2]
